<?php

class Easybill {
	
	var $method; // method used for exchanging data with the easybill server ('socket' or 'curl')
	
	function __construct( $auth_key, $method = 'socket' )
	{
		$this->auth_key = $auth_key;
		$this->method = $method;
		$this->server =  'soap.easybill.de';
		$this->path = 'soap.easybill.php';

		if ( function_exists( 'date_default_timezone_set' ) )
		{
			date_default_timezone_set( @date_default_timezone_get() );
		}

	}
	
	function get_customer( $customer_id )
	{
		$body = "<cus:GetCustomerRequest>{$customer_id}</cus:GetCustomerRequest>";
		$response = $this->_request( $body );
		if ( ! $response )
		{
			return FALSE;
		}
		$records = $this->xml_extract( $response, '.*', 'records' );
		return $records[0];
	}
	
	function get_customer_by_number( $customer_number )
	{
		$body = "<cus:GetCustomerByCustomerNumberRequest>{$customer_number}</cus:GetCustomerByCustomerNumberRequest>";
		$response = $this->_request( $body );
		if ( ! $response )
		{
			return FALSE;
		}
		$records = $this->xml_extract( $response, '.*', 'records' );
		return $records[0];
	}
	
	function search_customers( $searchstring )
	{
		$searchstring = htmlspecialchars( utf8_encode( $searchstring ) );
		$body = "<cus:SearchCustomersRequest>{$searchstring}</cus:SearchCustomersRequest>";
		$response = $this->_request( $body );
		if ( ! $response )
		{
			return FALSE;
		}
		return $this->xml_extract( $response, '.*?', 'records' );
	}
	
	function set_customer( $customer )
	{
		$customer = $this->_arr2xml( $customer );
		$body = "<cus:SetCustomerRequest>{$customer}</cus:SetCustomerRequest>";
		$response = $this->_request( $body );
		if ( ! $response )
		{
			return FALSE;
		}
		$records = $this->xml_extract( $response, 'customerID|customerNumber', 'records' );
		return $records[0];
	}
	
	function create_document( $document )
	{
		foreach( $document['documentPosition'] as $position )
		{
			$positions[] = '<documentPosition>' . $this->_arr2xml( $position ) . '</documentPosition>';
		}
		unset( $document['documentPosition'] );
		$document = $this->_arr2xml( $document );
		$positions = implode( '', $positions );
		$body = "<doc:CreateDocumentRequest>{$document}{$positions}</doc:CreateDocumentRequest>";
		$response = $this->_request( $body );
		if ( ! $response )
		{
			return FALSE;
		}
		$records = $this->xml_extract( $response, 'documentID|documentNumber', 'records' );
		return $records[0];
	}
	
	function _arr2xml( $array )
	{
		$result = '';
		foreach( $array as $name => $value )
		{
			$result .= "<{$name}>" . htmlspecialchars( $value, ENT_COMPAT | ENT_HTML401, 'UTF-8' ) . "</{$name}>";
		}
		return $result;
	}

	function _request( $body )
	{
		$request = <<< EOT
<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:web="http://www.easybill.de/webservice/" xmlns:cus="http://www.easybill.de/webservice/customer/" xmlns:doc="http://www.easybill.de/webservice/document/">
  <soapenv:Header>
     <web:UserAuthKey>{$this->auth_key}</web:UserAuthKey>
  </soapenv:Header>
  <soapenv:Body>
     {$body}
  </soapenv:Body>
</soapenv:Envelope>
EOT;
		// die( $request );
		
		switch ( $this->method )
		{
			case 'socket':
				return $this->_send_via_socket( $request );
			case 'curl':
				return $this->_send_via_curl( $request );
			default:
				return FALSE;
		}
	}

	function _send_via_socket( $request )
	{
		require_once dirname( __FILE__ ) . '/tcpconnection.class.php';
		require_once dirname( __FILE__ ) . '/httpconnection.class.php';
		$this->http = new httpconnection( $this->server, 443, TRUE );
		$this->response = $this->http->post( $this->path, array( $request ) );

		if ( $this->response['head']['code'] != 200 )
		{
			return FALSE;
		}

		return $this->response['body'];
	}

	function _send_via_curl( $request )
	{
		$curl = curl_init();
		curl_setopt( $curl, CURLOPT_URL, "https://{$this->server}/{$this->path}" );
		curl_setopt( $curl, CURLOPT_FORBID_REUSE, TRUE );
		curl_setopt( $curl, CURLOPT_FRESH_CONNECT, TRUE );
		curl_setopt( $curl, CURLOPT_RETURNTRANSFER, TRUE );
		curl_setopt( $curl, CURLOPT_SSL_VERIFYPEER, FALSE );
		curl_setopt( $curl, CURLOPT_POST, TRUE );
		curl_setopt( $curl, CURLOPT_POSTFIELDS, $request );
		$this->response = curl_exec( $curl );
		$info = curl_getinfo( $curl );
		curl_close( $curl );
		
		if ( $info['http_code'] != 200 )
		{
			return FALSE;
		}

		return $this->response;
	}
	
	function xml_extract( $xml, $element_match, $type = 'single' )
	{
		$matches = array();
		switch ( $type )
		{
			case 'single':
				$regex = '!<(' . $element_match . ')>([\s\S]*?)</\1>!';
				if ( ! preg_match( $regex, $xml, $matches ) ) return FALSE;
				return $matches[2];
				break;
			case 'all':
				$regex = '!<(' . $element_match . ')>([\s\S]*?)</\1>!';
				if ( ! preg_match_all( $regex, $xml, $matches, PREG_SET_ORDER ) ) return FALSE;
				foreach ( $matches as $item ) $result[] = array( $item[1] => $item[2] );
				return $result;
				break;
			case 'records':
				$regex = '!<(' . $element_match . ')>([^<]*)</\1>!';
				if ( ! preg_match_all( $regex, $xml, $matches, PREG_SET_ORDER ) ) return FALSE;
				$result = array();
				$record = array();
				$first_element = '';
				foreach ( $matches as $item )
				{
					if ( empty( $first_element ) ) $first_element = $item[1];
					elseif ( $item[1] == $first_element ){$result[] = $record; $record = array();}
					$record[ $item[1] ] = $item[2];
				}
				$result[] = $record;
				return $result;
				break;
			default:
				return FALSE;
		}
	}
}

// end of file easybill.class.php
