<?php

function makeThumbnail( $image, $force = FALSE )
{
	$source_file = v::vtglobal('pxtcPathHires') . $image;

	if ( v::vtglobal('pxtcImagingThumbnailSource') != 'hires'
			 || ! file_exists( $source_file ) )
	{
		$source_file = v::vtglobal('pxtcPathLores') . $image;
	}

	if ( ! file_exists( $source_file ) )
	{
		return -1; // Source file not found
	}

	$path = v::vtget('thumbnailPath') . dirname( $image );

	if ( file_exists( $path ) )
	{
		if ( ! is_dir( $path ) )
		{
			return -10; // Cannot create thumbnails directory
		}
	}
	else
	{
		if ( ! mkdir( $path, 0777, TRUE ) )
		{
			return -11; // Creation of thumbnails directory failed
		}
		
		chmod( $path, 0777 );
	}

	if ( ! is_writable( $path ) )
	{
		v::vtdo( array ( '-act' => 'ftp', '-cmd' => 'chmod', '-remote' => convertToFtpPath( $path ), '-mode' => '0777' ) );

		if ( v::vtresult() != 1 )
		{
			return -9; // Thumbnails directory not writable
		}
	}

	$path = v::vtget('thumbnailPath') . $image;

	if ( file_exists( $path ) && ! is_writable( $path ) )
	{
		v::vtdo( array ( '-act' => 'ftp', '-cmd' => 'delete', '-remote' => convertToFtpPath( $path ) ) );

		if ( v::vtresult() != 1 )
		{
			return -8; // Thumbnails file exists and cannot be overwritten
		}
	}

	if ( file_exists( $path ) && filemtime( $source_file ) <= filemtime( $path ) && ! $force )
	{
		return 0; // Thumbnails file is newer than source file
	}

	require_once 'templates/pixtacy/lib/image_processor.class.php';
	$imager = new Image_Processor();

	if ( ! $imager->load( $source_file ) )
	{
		return -2; // Not enough memory
	}

	$size = v::vtglobal('pxtcImagingThumbnailSize');
	$sizeMin = v::vtglobal('pxtcImagingThumbnailSizeMin');
	$quality = v::vtglobal('pxtcImagingThumbnailQuality');
	$result = $imager->resize( $size ? $size : 120, $sizeMin ? $sizeMin : 0 );

	if ( ! $imager->write( $path, $quality ? $quality : 80 ) )
	{
		return -12;
	}

	return 1;
}


function getIPTCHeader( $image, $header, $charset = '' )
{
	static $exif;

	$fieldNames = array (
		'Category' => '015',
		'SupCategory' => '020',
		'Keywords' => '025',
		'ReferenceNumber' => '050',
		'CreationDate' => '055',
		'CreationTime' => '060',
		'Photographer' => '080',
		'ByLine' => '080',
		'ByLineTitle' => '085',
		'City' => '090',
		'State' => '095',
		'CountryCode' => '100',
		'Country' => '101',
		'Credit' => '110',
		'Title' => '105',
		'Copyright' => '116',
		'Description' => '120',
		'Instructions' => '040',
		'Writer' => '122',
		'Rating' => 'xmp'
	);

	if ( !isset( $fieldNames[ $header ] ) )
	{
		return '';
	}

	if ( $fieldNames[ $header ] == 'xmp' )
	{
		if ( ! isset( $GLOBALS['pxtc']['xmp_data'][ $image ] ) )
		{
			error_reporting( E_ALL );
			require_once 'templates/pixtacy/lib/image/ImageInterface.php';
			require_once 'templates/pixtacy/lib/image/Image.php';
			require_once 'templates/pixtacy/lib/image/Metadata/Xmp.php';
			require_once 'templates/pixtacy/lib/image/Format/JPEG.php';
			require_once 'templates/pixtacy/lib/image/Format/JPEG/Segment.php';
			$imageObj = CSD\Image\Image::fromFile( $image );
			$GLOBALS['pxtc']['xmp_data'][ $image ] = $imageObj->getXmp();
		}

		if ( is_callable( array( $GLOBALS['pxtc']['xmp_data'][ $image ], "get{$header}" ) ) )
		{
			return call_user_func( array( $GLOBALS['pxtc']['xmp_data'][ $image ], "get{$header}" ) );
		}

		return '';
	}

	if ( isset( $GLOBALS['pxtc']['iptc_data'][ $image ] ) )
	{
		$iptc = $GLOBALS['pxtc']['iptc_data'][ $image ];
	}
	else
	{
		$GLOBALS['pxtc']['iptc_data'][ $image ] = array ();

		if ( !getimagesize( $image, $info ) )
		{
			return '';
		}

		if ( !isset( $info['APP13'] ) )
		{
			return '';
		}

		$iptc = iptcparse( $info['APP13'] );
		$GLOBALS['pxtc']['iptc_data'][ $image ] = $iptc;
	}

	$code = '2#' . $fieldNames[ $header ];

	if ( ! isset( $iptc[ $code ] ) )
	{
		if ( ( $header == 'CreationDate' || $header == 'CreationTime' )
				&& function_exists( 'exif_read_data' )
				&& ( isset( $exif[ $image ] ) || $exif[ $image ] = exif_read_data( $image ) )
				&& isset( $exif[ $image ]['DateTimeOriginal'] )
				&& substr_count( $exif[ $image ]['DateTimeOriginal'], ' ' ) == 1
				&& $exif[ $image ]['DateTimeOriginal'][0] != '0' )
		{
			list ( $date, $time ) = explode( ' ', $exif[ $image ]['DateTimeOriginal'] );
			return str_replace( ':', '', $header == 'CreationDate' ? $date : $time );
		} else {
			return '';
		}
	}

	if ( is_array( $iptc[ $code ] ) )
	{
		$result = implode( ', ', $iptc[ $code ] );
	}
	else
	{
		$result = isset( $iptc[ $code ] ) ? $iptc[ $code ] : '';
	}

	if ( $charset == 'utf-8' && function_exists('mb_convert_encoding') && defined('ENT_HTML401') ){
		return html_entity_decode( mb_convert_encoding( $result, 'HTML-ENTITIES', 'UTF-8' ), ENT_COMPAT | ENT_HTML401, 'UTF-8' );
	}
	elseif ( $charset )
	{
		require_once dirname(__FILE__) . '/ConvertCharset.class.php';
		$Coder = new ConvertCharset( $charset, 'utf-8', FALSE );
		return $Coder->Convert( $result );
	}
	else
	{
		return $result;
	}
}


function getDirectoryList( $path )
{
	if ( substr( $path, -1 ) != '/' )
	{
		$path .= '/';
	}

	$list = array ();
	$handle = opendir( $path );

	if ( $handle )
	{
		while ( $file = readdir( $handle ) )
		{
			if ( $file != '.' && $file != '..' && substr($file,0,1) != '.' )
			{
				if ( is_dir( $path.$file ) )
				{
					$list[] = $file;
				}
	 		}
		}

		closedir( $handle );
		sort( $list );
		return $list;
	}
	else
	{
    	return -1;
	}
}


function getDirectoryTree( $path, $tree = '', $basePath = '' )
{
	$handle = opendir( $path );

	if ( $handle )
	{
		if ( $basePath == '' )
		{
			$basePath = $path;
		}

		if ( $tree == '' )
		{
			$tree = '/';
		}
		else
		{
			$tree .= ', ' . substr( $path, strlen( $basePath ) );
		}

		while ( FALSE !== ( $item = readdir( $handle ) ) )
		{
			//echo $path.'/'.$item;
			if ( $item[0] != '.' && is_dir( $path.'/'.$item ) )
			{
				$tree = getDirectoryTree( $path.'/'.$item, $tree, $basePath );
	 		}
		}
		closedir( $handle );
		return $tree;
	} else {
    	return FALSE;
	}
}


function getFileList( $path, $pattern='.*' )
{
	if ( substr( $path, -1 ) != '/' )
	{
		$path .= '/';
	}

	$list = array ();
	$handle = opendir($path);

	if ( $handle )
	{
		while ( FALSE !== ( $file = readdir( $handle ) ) )
		{
			if ( $file != '.' && $file != '..' && substr( $file, 0, 1 ) != '.' )
			{
				if ( is_file( $path.$file ) && preg_match( '/' . $pattern . '/', $file ) > 0 )
				{
					$list[] = $file;
				}
	 		}
		}

		closedir( $handle );
		return $list;
	}
	else
	{
    	return -1;
	}

}


function getFileSize( $file )
{
	$units = array ( 'B', 'kB', 'MB', 'GB', 'TB' );
	$bytes = max( filesize( $file ), 0 );
	$pow = floor( ( $bytes ? log( $bytes ) : 0 ) / log( 1024 ) );
	$pow = min( $pow, count( $units ) - 1 );
	$bytes /= pow( 1024, $pow );
	return round( $bytes, $precision ) . ' ' . $units[ $pow ];
}


function getFileInfo( $file, $info )
{
	clearstatcache();
	$stats = stat( $file );
	return $stats[ $info ];
}


function imageName2URL( $str, $tolower = TRUE )
{
	// Ascii-Zeichen > 127 müssen in $str als HTML-Entites codiert sein!
	$str = trim( $str );

	if ( $tolower )
	{
		$str = strtolower( $str );
	}

	$search = array ( '&Auml;','&Ouml;','&Uuml;','&auml;','&ouml;','&uuml;','&szlig;','&eacute;','&egrave;','&ecirc;','&aacute;','&agrave;','&acirc;' );
	$replace = array ( 'Ae','Oe','Ue','ae','oe','ue','ss','e','e','e','a','a','a' );
	$str = str_replace( $search, $replace, $str );
	$str = preg_replace( '/[^a-zA-Z0-9-_@\.\s]/', '-', $str );
	$str = preg_replace( '/-+/', '-', $str );
	return $str;
}


function encodeNumberList( $list )
{
	$arr = array_unique( explode( ',', $list ) );
	$result = array ();

	foreach ( $arr as $item )
	{
		if ( $item > 0 )
		{
			$result[] = base_convert( $item, 10, 35 );
		}
	}

	return implode( 'z', $result );
}


function decodeNumberList( $list )
{
	$arr = explode( 'z', $list );
	$result = array ();

	foreach ( $arr as $item )
	{
		$result[] = base_convert( $item, 35, 10 );
	}

	return implode( ',', $result );
}


function isImageInCart( $imageId )
{
	$cart = json_decode( v::vtget('Cart'), TRUE );

	if ( ! is_array( $cart ) )
	{
		return '';
	}

	foreach ( $cart as $position )
	{
		if ( $position['image'] == $imageId )
		{
			return '1';
		}
	}

	return '';
}


function countList( $jList )
{
	if ( trim( $jList ) == '' )
	{
		return 0;
	}

	if ( $jList[0] == '[' )
	{
		$list = json_decode( $jList, TRUE );
	}
	else
	{
		$list = array_filter( explode( ',', $jList ) );
	}

	return count( $list );
}


function evalList( $jList, $attribute, $operator = 'sum' )
{
	if ( $jList )
	{
		$list = json_decode( $jList, TRUE );
		$result = 0;

		foreach ( $list as $item )
		{
			switch ( $operator )
			{
				case 'max' :
					$result = max( $item[ $attribute ], $result );
					break;
				case 'sum' :
					$result += str_replace( ',', '.', $item[ $attribute ]);
					break;
			}
		}

		return $result;
	}

	return FALSE;
}


function createDownload( $path, $reference, $expirationDays = 1 )
{
	if ( substr( $path, 0, 1 ) == '/' && !file_exists( $path ) )
	{
		return '';
	}

	$db = v::get_db();
	$path = $db->escape_string( $path );
	$query = "CREATE TABLE IF NOT EXISTS `%table_downloads` (
		`code` varchar(32) NOT NULL default '',
		`path` text default '',
		`reference` varchar(64) NOT NULL default '',
		`created` datetime DEFAULT NULL,
		`expires` datetime DEFAULT NULL,
		`download_ts` datetime DEFAULT NULL,
		`download_ip` varchar(15) NOT NULL default '',
		PRIMARY KEY  (`code`)
		)";
	v::query( $query );
	do {
		$code = md5( rand( 1000, 9999 ) . microtime() );
		$query = "INSERT INTO `%table_downloads` (`code`,`path`,`reference`,`created`,`expires`) VALUES ('$code','$path','$reference', NOW(), DATE_ADD( CURRENT_TIMESTAMP, INTERVAL $expirationDays DAY ) )";
		$result = v::query( $query );
	}
	while ( $result && ! $db->affected_rows );

	return $code;
}


function deleteDownloads( $reference )
{
	if ( $reference )
	{
		$query = "DELETE FROM `%table_downloads` WHERE reference = '$reference'";
		v::query( $query );
	}
}


function getImageOrientation( $file, $precision = 0 )
{
	if ( ! file_exists( $file ) )
	{
		return '';
	}

	list ( $width, $height, $type, $attr ) = getimagesize( $file );

	if ( $precision > 0 && abs( $height - $width ) / min( $height, $width ) < $precision )
	{
		return 'square';
	}

	return $height > $width ? 'portrait' : 'landscape';
}


function getImageLongSide( $file )
{
	if ( ! file_exists( $file ) )
	{
		return '';
	}

	list ( $width, $height, $type, $attr ) = getimagesize( $file );

	if ( $height < $width )
	{
		return $width;
	}

	return $height;
}


function getImageInfo( $file, $precision = 0 )
{
	if ( ! file_exists( $file ) )
	{
		return '';
	}

	list ( $width, $height ) = getimagesize( $file );

	if ( $precision > 0 && abs( $height - $width ) / min( $height, $width ) < $precision )
	{
		$orientation = 'square';
	}
	else
	{
		$orientation = $height > $width ? 'portrait' : 'landscape';
	}

	return '{"width":' . $width . ', "height":' . $height
			. ', "orientation":"' . $orientation . '"'
			. ', "broadside":"' . ( $height < $width ? 'width' : 'height' ) . '"'
			. ', "broadlength":' . ( $height < $width ? $width : $height ) . '}';
}


function getInputHashSU()
{
	if ( !$_GET['-ses'] )
	{
		$query = html_entity_decode( $_SERVER['QUERY_STRING'] );
		parse_str( $query, $_GET );
	}

	$data = array (
		'user_id' => v::vtglobal('pxtcSofubUserID'),
		'project_id' => v::vtglobal('pxtcSofubProjectID'),
		'sender_holder' => '',
		'sender_account_number' => '',
		'sender_bank_code' => '',
		'sender_country_id' => '',
		'amount' => v::vtget('Gesamtpreis|num-en:2.'),
		'currency_id' => 'EUR',
		'reason_1' => 'Bestell-Nr. ' . v::vtsession('OrderNumber'),
		'reason_2' => '',
		'user_variable_0' => $_GET['-pg'],
		'user_variable_1' => $_GET['-ses'],
		'user_variable_2' => $_GET['-cod'],
		'user_variable_3' => v::vtsession('OrderPageID'),
		'user_variable_4' => '',
		'user_variable_5' => '',
		'project_password' => v::vtglobal('pxtcSofubPassword')
	);
	$data_implode = implode( '|', $data );
	$hash = md5( $data_implode );
	return $hash;
}


function getConfirmationHashSU()
{
	if ( v::vtglobal('pxtcSofubNotificationPassword') != '' )
	{
		$password = v::vtglobal('pxtcSofubNotificationPassword');
	}
	else
	{
		$password = v::vtglobal('pxtcSofubPassword');
	}

	$data = array (
	 'transaction' => $_POST['transaction'],
	 'user_id' => $_POST['user_id'],
	 'project_id' => $_POST['project_id'],
	 'sender_holder' => $_POST['sender_holder'],
	 'sender_account_number' => $_POST['sender_account_number'],
	 'sender_bank_code' => $_POST['sender_bank_code'],
	 'sender_bank_name' => $_POST['sender_bank_name'],
	 'sender_bank_bic' => $_POST['sender_bank_bic'],
	 'sender_iban' => $_POST['sender_iban'],
	 'sender_country_id' => $_POST['sender_country_id'],
	 'recipient_holder' => $_POST['recipient_holder'],
	 'recipient_account_number' => $_POST['recipient_account_number'],
	 'recipient_bank_code' => $_POST['recipient_bank_code'],
	 'recipient_bank_name' => $_POST['recipient_bank_name'],
	 'recipient_bank_bic' => $_POST['recipient_bank_bic'],
	 'recipient_iban' => $_POST['recipient_iban'],
	 'recipient_country_id' => $_POST['recipient_country_id'],
	 'international_transaction' => $_POST['international_transaction'],
	 'amount' => $_POST['amount'],
	 'currency_id' => $_POST['currency_id'],
	 'reason_1' => $_POST['reason_1'],
	 'reason_2' => $_POST['reason_2'],
	 'security_criteria' => $_POST['security_criteria'],
	 'user_variable_0' => $_POST['user_variable_0'],
	 'user_variable_1' => $_POST['user_variable_1'],
	 'user_variable_2' => $_POST['user_variable_2'],
	 'user_variable_3' => $_POST['user_variable_3'],
	 'user_variable_4' => $_POST['user_variable_4'],
	 'user_variable_5' => $_POST['user_variable_5'],
	 'created' => $_POST['created'],
	 'password' => $password
	);
	$data_implode = implode( '|', $data );
	$hash = md5( $data_implode );
	return $hash;
}


function verifyPaypalIPNMessage()
{
	$req = 'cmd=_notify-validate';

	foreach ( $_POST as $key => $value )
	{
		$value = urlencode( $value );
		$req .= "&$key=$value";
	}

	// Post back to PayPal to validate
	$host = isset( $_POST['test_ipn'] ) ? 'www.sandbox.paypal.com' : 'www.paypal.com';
	$header = "POST /cgi-bin/webscr HTTP/1.1\r\n";
	$header .= "Content-Type: application/x-www-form-urlencoded\r\n";
	$header .= "Host: {$host}\r\n";
	$header .= "Content-Length: " . strlen( $req ) . "\r\n";
	$header .= "Connection: close\r\n\r\n";
	$fp = fsockopen( "ssl://{$host}", 443, $errno, $errstr, 30 );

	if ( ! $fp )
	{
		return 'ERROR 1';
	}

	fwrite ( $fp, $header . $req );
	$res = '';

	while ( ! feof( $fp ) )
	{
	  $res .= fgets( $fp, 128 );
	}

	fclose( $fp );
	$parts = explode( "\r\n\r\n", trim( $res ) );
	$res = trim( end( $parts ) );
	return strpos( $res, 'VERIFIED' ) !== FALSE ? 'VERIFIED' : 'INVALID';
}


function createPassword( $length = 10 )
{
	$characters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_.:,;#+*?=()/$!';

	if ( $length > 0 && $length < 50 )
	{
		$pw = '';

		while ( strlen( $pw ) < $length )
		{
			$pw .= $characters[ rand( 0, strlen( $characters ) - 1 ) ];
		}

		return $pw;
	}

	return FALSE;
}


function analyzeSearchString( $search = '' )
{
	if ( empty( $search ) )
	{
		$search = $_REQUEST['-search'];
	}

	if ( empty( $search ) )
	{
		return '';
	}

	if ( is_array( $search ) )
	{
		$search = implode( ' ', $search );
	}

	$matches = array ();
	$result['all'] = '';
	preg_match_all( '/-?([a-zA-Z0-9_]+:)?(("[^"]+")|([\S]+))/', $search, $matches );

	foreach ( $matches[0] as $criteria )
	{
		if ( preg_match( '/^(-?[a-zA-Z0-9_]+):/', $criteria, $field ) )
		{
			if ( empty( $result[ strtolower( $field[1] ) ] ) )
			{
				$result[ strtolower( $field[1] ) ] = substr( $criteria, strlen( $field[1] ) + 1 ) . ' ';
			}
			else
			{
				$result[ strtolower( $field[1] ) ] .= substr( $criteria, strlen( $field[1] ) + 1 ) . ' ';
			}
		}
		else
		{
			$result['all'] .= $criteria . ' ';
		}
	}
	
	$scope = v::vtglobal('pxtcSearchScope');

	if ( $scope != '' && empty( $result[ $scope ] ) )
	{
		$result[ $scope ] = $result['all'];
		unset( $result['all'] );
	}
	
	$fields = v::vtglobal('pxtcSearchAllowedFields');

	if ( $fields )
	{
		$fields = array_map( 'trim', explode( ',', strtolower( $fields ) ) );
		$fields = array_flip( $fields ) + array ( $scope => $result[ $scope ] );
		$result = array_intersect_key( $result, $fields );
	}

	if ( v::vtglobal('pxtcSearchFtMode') == '' )
	{
		$result = array_map( function( $v ){ return str_replace( '*', '%', $v ); }, $result );
	}
	
	return $result;
}


function isZipAvailable()
{
	$result = 0;

	if ( function_exists('exec') )
	{
		$cmd = v::vtconfig('pathToZipCommand') ? v::vtconfig('pathToZipCommand') : '/usr/bin/zip';
		exec( $cmd . ' -L', $output );

		if ( strpos( implode( $output ), 'Copyright' ) !== FALSE )
		{
			$result += 1;
		}
	}

	if ( function_exists( 'hash_file' ) )
	{
		$result += 2;
	}

	return $result;
}


function getZipMethod()
{
	$zip = isZipAvailable();

	if ( ! $zip  )
	{
		return 'deaktiviert';
	}

	if ( $zip == 1 || $zip == 3 && v::vtglobal('pxtcForceExternalZip') == '1' )
	{
		return 'external';
	}

	return 'internal';
}


function checkEnvironment()
{
	$result = array ();

	if ( !function_exists( 'imagecreatetruecolor' ) )
	{
		$result[] = 'In der PHP-Umgebung steht keine GD-Library zur Verfügung. Sprechen Sie Ihren Provider oder Administrator an, damit diese nachinstalliert wird.';
	}

	$thumbnailsDir = v::vtconfig('pathToData') . 'pixtacy_thumbnails';

	if ( ! file_exists( $thumbnailsDir ) )
	{
		mkdir( $thumbnailsDir );
		chmod( $thumbnailsDir, 0777 );
	}

	if ( ! file_exists( $thumbnailsDir ) )
	{
		$result[] = 'Thumbnails-Verzeichnis konnte nicht erstellt werden. Bitte erweitern Sie die Zugriffsrechte des \'data\'-Verzeichnisses mit Hilfe Ihres FTP-Programms (chmod 777).';
	}
	else
	{
		$tempDir = $thumbnailsDir . '/.temp';

		if ( file_exists( $tempDir ) )
		{
			if ( ! rmdir( $tempDir ) )
			{
				$result[] = "Temporäres Verzeichnis {$tempDir} konnte nicht gelöscht werden. Bitte entfernen Sie dieses mit Hilfe Ihres FTP-Programms.";
			}
			else
			{
				mkdir( $tempDir, 0777 );

				if ( !file_exists( $tempDir ) )
				{
					$result[] = 'Erstellen eines Unterverzeichnisses ist fehlgeschlagen. Vermutlich ist auf dem Server der PHP-Safe-mode aktiviert. Sprechen Sie Ihren Provider oder Administrator an, damit dieser abgeschaltet wird.';
				}
				else
				{
					$testFile = $tempDir . '/test';

					if ( ! $fp = fopen( $testFile, 'w' ) )
					{
						$result[] = 'Erzeugen einer Testdatei ist fehlgeschlagen. Vermutlich ist auf dem Server der PHP-Safe-mode aktiviert. Sprechen Sie Ihren Provider oder Administrator an, damit dieser abgeschaltet wird.';
					}
					else
					{
						fclose( $fp );
						unlink( $testFile );
					}

					rmdir( $tempDir );
				}
			}
		}
	}

	return implode( '|', $result );
}


function checkDirectory( $dir, $create = FALSE )
{
	// prüft, ob das Verzeichnis $dir existiert und erstellt es, falls $create gesetzt ist
	// $dir muss relativ zum lores-Verzeichnis angegeben sein

	if ( $dir == '' )
	{
		return -1;
	}
	elseif ( $dir == '/' )
	{
		return 1;
	}
	elseif ( $dir[0] == '/' && file_exists( v::vtglobal('pxtcPathLores') . $dir ) )
	{
		return 1;
	}
	elseif ( ! $create )
	{
		return -1;
	}

	if ( v::vtglobal('pxtcLocalAccess') == 'direct' )
	{
		if ( ! checkLocalDirectory( v::vtconfig('pathToVirthos') . v::vtglobal('pxtcPathLores') . $dir ) )
		{
			return -8; // lokales Verzeichnis konnte nicht erstellt werden oder ist nicht beschreibbar
		}

		if ( v::vtconfig('pxtcCreateHiresFolders') )
		{
			checkLocalDirectory( v::vtconfig('pathToVirthos') . v::vtglobal('pxtcPathHires') . $dir );
		}

		return 1;
	}

	if ( v::ftpConnect() < 0 )
	{
		return -3; // keine Verbindung zum FTP-Server möglich
	}

	if ( ! $loresPath = convertToFtpPath( v::vtglobal('pxtcPathLores') ) )
	{
		return -4; // Angabe zum lores-Verzeichnis fehlerhaft
	}

	v::vtdo( array ( '-act' => 'ftp', '-cmd' => 'chdir', '-remote' => $loresPath ) );

	if ( v::vtresult() != 1 )
	{
		return -5; // kein Wechsel zum lores-Verzeichnis möglich
	}

	$dir = trim( $dir, '/' );

	v::vtdo( array ( '-act' => 'ftp', '-cmd' => 'chdir', '-remote' => $dir ) );

	if ( v::vtresult() != 1 )
	{
		if ( $create )
		{
			v::vtdo( array ( '-act' => 'ftp', '-cmd' => 'mkdir', '-remote' => $dir ) );
			if ( v::vtresult() != 1 )
			{
				return -6; // Erstellen des Verzeichnisses $dir fehlgeschlagen
			}
		}
		else
		{
			return -7; // Verzeichnis $dir existiert nicht
		}
	}

	return 1;
}


function checkLocalDirectory( $dir )
{
	if ( $dir == '' || $dir[0] != '/' || strpos( $dir, '/../' ) !== FALSE )
	{
		return -1;
	}

	if ( file_exists( $dir ) )
	{
		if ( is_writable( $dir ) )
		{
			return 1;
		}

		$remotePath = v('vtFtpBasePath') . '/' . substr( $dir, strlen( v::vtconfig('pathToVirthos') ) );
		v::vtdo( array ( '-act' => 'ftp', '-cmd' => 'chmod', '-remote' => $remotePath, '-mode' => '777' ) );
		clearstatcache();
		return is_writable( $dir ) ? 1 : -2;
	}

	if ( ! mkdir( $dir, 0777, TRUE ) )
	{
		return -3;
	}

	chmod( $dir, 0777 );
	return 1;
}


function convertToFtpPath( $path )
{
	// konvertiert $path zu einem FTP-Pfad
	if ( $path[0] == '/' )
	{
		return FALSE; // absolute Pfade sind nicht erlaubt
	}

	return rtrim( v::vtget( 'ftpBasePath' ), '/' ) . '/' . $path;
}


function getFtpPaths( $path, $is_dir = FALSE )
{
	// liefert eine Liste mit absoluten FTP-Pfaden zu allen Bilddateien bzw. -verzeichnissen,
	// die zu $path gehören (lores, hires, thumbnails)
	// $path ist eine Pfadangabe relativ zum lores-Verzeichnis

	$result[] = convertToFtpPath( 'data/pixtacy_thumbnail' ) . $path;

	if ( $loresPath = v::vtglobal('pxtcPathLores') )
	{
		$result[] = convertToFtpPath( $loresPath ) . $path;
	}

	if ( $hiresPath = v::vtglobal('pxtcPathHires') )
	{
		if ( $is_dir )
		{
			$result[] = convertToFtpPath( $hiresPath ) . $path;
		}
		else
		{
			$list = getHiresFiles( $path );

			if ( ! empty( $list ) )
			{
				$hiresPath = convertToFtpPath( $hiresPath ) . dirname( $path );

				foreach ( $list as $item )
				{
					$result[] = cleanPath( $hiresPath . '/' . $item['directory'] . '/' . $item['name'] );
				}
			}
		}
	}

	return $result;
}


function ftpDelete( $path, $is_dir = FALSE )
{
	// löscht die Dateien, die zu einem Bild bzw. zu einer Galerie gehören, vom Server
	// $path muss relativ zum lores-Verzeichnis angegeben sein

	if ( $path == '' || $path == '/' || strpos( $path, '../' ) !== FALSE )
	{
		return FALSE;
	}

	if ( v::ftpConnect() < 0 )
	{
		return -1; // keine Verbindung zum FTP-Server möglich
	}

	$list = getFtpPaths( $path, $is_dir );

	if ( empty( $list ) )
	{
		return FALSE;
	}

	// Objekte löschen
	$counter = 0;

	foreach ( $list as $item )
	{
		v::vtdo( array ( '-act' => 'ftp', '-cmd' => 'delete', '-remote' => $item, '-recursive' => '1' ) );

		if ( v::vtresult() == 1 )
		{
			$counter++;
		}
	}

	return $counter > 0 ? 1 : 0;
}


function ftpDeleteThumbs( $path )
{
	// löscht zu einem Bild bzw. zu einer Galerie die zugehörigen Thumbnails vom Server
	// $path muss relativ zum lores-Verzeichnis angegeben sein

	if ( $path == '' || $path == '/' || strpos( $path, '../' ) !== FALSE )
	{
		return FALSE;
	}

	if ( v::ftpConnect() < 0 )
	{
		return -1; // keine Verbindung zum FTP-Server möglich
	}

	v::vtdo( array (
					'-act' => 'ftp',
					'-cmd' => 'delete',
					'-remote' => convertToFtpPath( 'data/pixtacy_thumbnail' ) . $path,
					'-recursive' => '1'
					)
				);

	return v::vtresult();
}


function getHiresFiles( $image )
{
	// liefert eine Liste mit allen Hires-Dateien, die zum Bild $image existieren
	$name = basename( $image );
	$path = v::vtglobal('pxtcPathHires') . '/' . trim( dirname( $image ), '/' ) . '/';
	$folderlist = getDirectoryList( $path );

	if ( $folderlist == -1 )
	{
		return '';
	}

	array_unshift( $folderlist, '/' );
	$result = array ();

	foreach ( $folderlist as $folder )
	{
		$filelist = getFileList( $path . $folder, '^' . preg_quote( substr( $name, 0, strrpos( $name, '.' ) ) ) . '\..+' );

		if ( $filelist != '-1' )
		{
			foreach ( $filelist as $file )
			{
				$filepath = $path . $folder . '/' . $file;
				$imagesize = getimagesize( $filepath );
				$result[] = array (
					'name' => $file,
					'directory' => $folder == '/' ? '' : $folder,
					'size' => getFileSize( $filepath ),
					'extension' => strtolower( pathinfo( $filepath, PATHINFO_EXTENSION ) ),
					'resolution' => $imagesize ? implode( ' x ', array_slice( $imagesize, 0, 2 ) ) : ''
				);
			}
		}
	}
	return empty( $result ) ? '' : $result;
}


function cleanPath( $path, $removeTrailingSlash = FALSE )
{
	$path = preg_replace( '|/+|', '/', $path );

	if ( $removeTrailingSlash && substr( $path, -1 ) == '/' )
	{
		$path = substr( $path, 0, -1 );
	}

	return $path;
}


function transferToPxprintNet( $userId, $items, $key, $contact, $delivery, $subject = '' )
{
	$timestamp = time();
	$contact = json_decode( $contact, TRUE );
	$request = 'shop_id=' . $userId
		. '&subject=' . urlencode( $subject )
		. '&items=' . urlencode( $items ) 
		. '&timestamp=' . $timestamp
		. '&hash=' . md5( $userId . $items . $timestamp . $key ) 
		. '&base_url=' . ( empty( $_SERVER['HTTPS'] ) || $_SERVER['HTTPS'] == 'off' ? 'http' : 'https' )
		. '://' . $_SERVER['HTTP_HOST'] . dirname( $_SERVER['SCRIPT_NAME'] )
		. '/templates/pixtacy/modules/download.php?c='
		. '&inv_lastname=' . urlencode( $contact['lastname'] )
		. '&inv_firstname=' . urlencode( $contact['firstname'] )
		. '&inv_organization=' . urlencode( $contact['organization'] )
		. '&inv_address=' . urlencode( $contact['address'] )
		. '&inv_city=' . urlencode( $contact['city'] )
		. '&inv_zipcode=' . urlencode( $contact['zipcode'] )
		. '&inv_country=' . urlencode( $contact['country'] )
		. '&inv_state=' . urlencode( $contact['state'] )
		. '&inv_phone=' . urlencode( $contact['phone'] )
		. '&inv_email=' . urlencode( $contact['email'] )
		. '&ship_to_enduser=' . ( $delivery == 'SHOP' ? '0' : '1' );

	if ( ! empty( $delivery ) && $delivery != 'SHOP' )
	{
		$delivery = json_decode( $delivery, TRUE );
		$request .= '&del_lastname=' . urlencode( $delivery['lastname'] )
			. '&del_firstname=' . urlencode( $delivery['firstname'] )
			. '&del_organization=' . urlencode( $delivery['organization'] )
			. '&del_address=' . urlencode( $delivery['address'] )
			. '&del_city=' . urlencode( $delivery['city'] )
			. '&del_zipcode=' . urlencode( $delivery['zipcode'] )
			. '&del_country=' . urlencode( $delivery['country'] )
			. '&del_state=' . urlencode( $delivery['state'] );
	}

	$header = "POST /index.php/api/create HTTP/1.0\r\n"
 		. "Host: api.pxprint.net\r\n"
		. "Content-Type: application/x-www-form-urlencoded\r\n"
		. "Content-Length: " . strlen( $request ) . "\r\n"
		. "Connection: close\r\n\r\n";

	$fp = fsockopen( 'api.pxprint.net', 80, $errno, $errstr, 30 );

	if ( ! $fp )
	{
		return '-1';
	}

	fwrite( $fp, $header . $request );

	while ( ! feof( $fp ) )
	{
	  $result .= fgets( $fp, 128 );
	}

	fclose( $fp );
	$lines = explode( "\n", $result );
	end( $lines );

	if ( strpos( $lines[0], '200' ) > 0 )
	{
		return current( $lines );
	}

	return '-2 ' . current( $lines );
}


function chmodr( $path, $filemode )
{
	if ( ! is_dir( $path ) )
	{
		return @chmod( $path, $filemode );
	}

	$dh = opendir( $path );

	while ( ( $file = readdir( $dh ) ) !== FALSE )
	{
		if ( $file != '.' && $file != '..' )
		{
			$fullpath = $path . '/' . $file;

			if ( is_link( $fullpath ) )
			{
				return FALSE;
			}

			if ( ! is_dir( $fullpath ) && ! @chmod( $fullpath, $filemode ) )
			{
				return FALSE;
			}

			if ( ! chmodr( $fullpath, $filemode ) )
			{
				return FALSE;
			}
		}
	}

	closedir( $dh );

	if ( @chmod( $path, $filemode ) )
	{
		return TRUE;
	}

	return FALSE;
}


function createEasybillInvoice( $customer_id,  $customer_number )
{
	if ( function_exists('my_create_invoice') )
	{
		return my_create_invoice( $customer_id, $customer_number );
	}

	if ( ! checkSecureTransferMethod() )
	{
		return -1; // der Server erlaubt keine sicheren Verbindungen zu anderen Diensten
	}

	$easybill = new Easybill( v::vtglobal('pxtcEasybillApiKey'), v::vtglobal('pxtcSecureTransferMethod') );
	$customer = array ();

	### ggf. vorhandenen Kundendatensatz holen ###

	do
	{
		if ( $customer_id > 0 )
		{
			$response = $easybill->get_customer( $customer_id );
			if ( $response != FALSE )
			{
				$customer = $response;
				break;
			}
		}

		if ( $customer_number > 0 )
		{
			$response = $easybill->get_customer_by_number( $customer_number );
			if ( $response != FALSE )
			{
				$customer = $response;
				break;
			}
		}

		$response = $easybill->search_customers( v::vtpage( 'Name' ) );

		if ( $response == FALSE )
		{
			break;
		}

		foreach ( $response as $record )
		{
			if ( strtolower( $record['firstName'] ) == strtolower( v::vtpage( 'Vorname' ) )
					&& strtolower( $record['lastName'] ) == strtolower( v::vtpage( 'Name' ) )
					&& strtolower( $record['companyName'] ) == strtolower( v::vtpage( 'Firma' ) )
					&& strtolower( $record['street'] ) == strtolower( v::vtpage( 'Strasse' ) )
					&& strtolower( $record['zipCode'] ) == strtolower( v::vtpage( 'PLZ' ) )
					&& strtolower( $record['city'] ) == strtolower( v::vtpage( 'Ort' ) )
					&& strtolower( $record['country'] ) == strtolower( v::vtpage( 'Land' ) )
					&& strtolower( $record['email'] ) == strtolower( v::vtpage( 'Email' ) ) )
			{
				$customer = $record;
				break;
			}
		}
	}
	while ( FALSE );

	### Kundendatensatz erstellen bzw. aktualisieren ###

	$customer['salutation'] = v::vtpage( 'Anrede' );
	$customer['firstName'] = v::vtpage( 'Vorname' );
	$customer['lastName'] = v::vtpage( 'Name' );
	$customer['companyName'] = v::vtpage( 'Firma' );
	$customer['street'] = v::vtpage( 'Strasse' );
	$customer['zipCode'] = v::vtpage( 'PLZ' );
	$customer['city'] = v::vtpage( 'Ort' );
	$customer['country'] = v::vtpage( 'Land' );
	$customer['ustid'] = v::vtpage( 'USt_ID' );
	$customer['email'] = v::vtpage( 'Email' );
	$customer['phone_1'] = v::vtpage( 'Telefon' );

	$response = $easybill->set_customer( $customer );

	if ( empty( $response['customerID'] ) )
	{
		return array ( 'error' => 'Kundendatensatz konnte nicht erstellt oder aktualisiert werden' );
	}

	$customer = $response;

	### Rechnungspositionen zusammenstellen ###

	$items = json_decode( v::vtpage( 'Warenkorb' ), TRUE );
	$products = array ();
	$max_vat_rate = 0;
	$vatExcluded = v::vtglobal('pxtcVatHandling') == 'excluded';
	foreach ( $items as $item )
	{
		if ( isset( $products[ $item['product'] ] ) )
		{
			$products[ $item['product'] ]['count'] += $item['quantity'];
		}
		else
		{
			$products[ $item['product'] ] = array
				(
					'positionType' => 'POSITION',
					'itemNumber' => $item['product'],
					'itemDescription' => $item['title'],
					'count' => $item['quantity'],
					'unit' => '',
					'singlePriceNetto' => 100 * ( $vatExcluded ? $item['netprice'] : $item['price'] / ( 1 + $item['vatrate']/100 ) ),
					'ustPercent' => $item['vatrate']
				);
		}
		$max_vat_rate = $item['vatrate'] > $max_vat_rate ? $item['vatrate'] : $max_vat_rate;
	}

	### ggf. Rabatt hinzufügen ###

	if ( v::vtpage( 'Rabatt_netto' ) > 0 )
	{
		$products[] = array
			(
				'positionType' => 'POSITION',
				'itemDescription' => v::vtget( 's_fld_discount' ),
				'count' => 1,
				'unit' => '',
				'singlePriceNetto' => -100 * v::vtpage( 'Rabatt_netto' ),
				'ustPercent' => $max_vat_rate
			);
	}

	### ggf. Preis für CD-Versand hinzufügen ###

	if ( v::vtpage( 'CD_Versand_netto' ) > 0 )
	{
		$products[] = array
			(
				'positionType' => 'POSITION',
				'itemDescription' => v::vtglobal( 'pxtcDeliveryDownloadablesText2de' ),
				'count' => 1,
				'unit' => '',
				'singlePriceNetto' => 100 * v::vtpage( 'CD_Versand_netto' ),
				'ustPercent' => $max_vat_rate
			);
	}

	### ggf. Auftragspauschale hinzufügen ###

	if ( v::vtpage( 'Auftragspauschale_netto' ) > 0 )
	{
		$products[] = array
			(
				'positionType' => 'POSITION',
				'itemDescription' => v::vtget( 's_fld_lumpsum' ),
				'count' => 1,
				'unit' => '',
				'singlePriceNetto' => 100 * v::vtpage( 'Auftragspauschale_netto' ),
				'ustPercent' => $max_vat_rate
			);
	}

	### ggf. Versandkosten hinzufügen ###

	if ( v::vtpage( 'Versandkosten_netto' ) > 0 )
	{
		$products[] = array
			(
				'positionType' => 'POSITION',
				'itemDescription' => v::vtget( 's_fld_shipping_costs' ) . ' (' . v::vtpage( 'Liefermethode' ) . ')',
				'count' => 1,
				'unit' => '',
				'singlePriceNetto' => 100 * v::vtpage( 'Versandkosten_netto' ),
				'ustPercent' => $max_vat_rate
			);
	}

	### Rechnungsdokument erstellen ###

	$document['customerID'] = $customer['customerID'];
	$document['documentType'] = 'INVOICE';
	$document['currency'] = v::vtglobal('pxtcEasybillCurrency');
	$document['contactLabel'] = v::vtget('s_fld_order_number');
	$document['contactText'] = v::vtpage('Nummer');
	$document['documentPosition'] = array_values( $products );
	$document['templateName'] = v::vtglobal('pxtcEasybillTemplate_' . v::vtpage('Language') );
	$document['text'] = v::vtget( 's_invoice_text_' . strtolower( v::vtpage('Bezahlmethode') ) );
	$document['sendasemail'] = ( v::vtglobal( 'pxtcEasybillAutoSend' ) == '1' );

	if ( v::vtpage('nicht_steuerbar') == '1' )
	{
		$document['taxOptions'] = v::vtpage('USt_ID') == '' ? 'nStb' : 'nStbUstID';
	}

	$response = $easybill->create_document( $document );

	if ( empty( $response['documentID'] ) )
	{
		return array ( 'error' => 'Rechnung konnte nicht erstellt werden' );
	}

	$result = array (
		'document_id' => $response['documentID'],
		'document_number' => $response['documentNumber'],
		'customer_id' => $customer['customerID'],
		'customer_number' => $customer['customerNumber']
		);

	return $result;

}


function checkSecureTransferMethod( $force = FALSE )
{

	if ( in_array( v::vtglobal('pxtcSecureTransferMethod'), array ( 'socket', 'curl' ) )
			&& ! $force )
	{
		return v::vtglobal('pxtcSecureTransferMethod');
	}

	if ( function_exists( 'fsockopen' ) )
	{
		$fp = @fsockopen( "ssl://www.google.com", 443 );
		if ( $fp )
		{
			fclose( $fp );
			v::vtdo( array ( '-act' => 'update', '-obj' => 70, 'pxtcSecureTransferMethod' => 'socket' ) );
			return 'socket';
		}
	}

	if ( function_exists( 'curl_init' ) )
	{
		$curl = curl_init();
		curl_setopt( $curl, CURLOPT_URL, "https://www.google.com/" );
		curl_setopt( $curl, CURLOPT_HEADER, 0 );
		curl_setopt( $curl, CURLOPT_RETURNTRANSFER, 1 );
		$result = curl_exec( $curl );
		curl_close($curl);
		if ( $result !== FALSE )
		{
			v::vtdo( array ( '-act' => 'update', '-obj' => 70, 'pxtcSecureTransferMethod' => 'curl' ) );
			return 'curl';
		}
	}

	return FALSE;
}


function check_vat_number( $number, $country = '' )
{
	$codes = 'AT|BE|BG|CY|CZ|DE|DK|EE|EL|ES|FI|FR|GB|HU|IE|IT|LT|LU|LV|MT|NL|PL|PT|RO|SE|SI|SK';

	$number = str_replace( ' ', '', $number );

	if ( $number == '' || strlen( $number ) < 10 || strlen( $number ) > 15 )
	{
		return FALSE;
	}

	if ( $country == '' )
	{
		if ( ! in_array( substr( $number, 0, 2 ), explode( '|', $codes ) ) )
		{
			return FALSE;
		}
	}
	else
	{
		if ( $country == 'GR' )
		{
			$code = 'EL';
		}
		else
		{
			$code = strtoupper( $country );
		}

		if ( substr( $number, 0, 2 ) != $code )
		{
			return FALSE;
		}
	}

	return TRUE;
}


function addCleverreachReceiver( $email, $lastname = '', $firstname = '', $salutation = '' )
{
	if ( ! checkSecureTransferMethod() && ! class_exists( 'SoapClient' ) )
	{
		return -1; // der Server erlaubt keine sicheren Verbindungen zu anderen Diensten
	}

	if ( class_exists( 'SoapClient' ) )
	{
		$api = new SoapClient('http://api.cleverreach.com/soap/interface_v5.1.php?wsdl');
		$user = array (
			'email' => $email,
			'registered' => time(),
			'activated' => time(),
			'source' => v::vtglobal('siteName'),
			'attributes' => array (
			array ( 'key' => 'firstname', 'value' => $firstname ),
			array ( 'key' => 'vorname', 'value' => $firstname ),
			array ( 'key' => 'lastname', 'value' => $lastname ),
			array ( 'key' => 'nachname', 'value' => $lastname ),
			array ( 'key' => 'salutation', 'value' => $salutation ),
			array ( 'key' => 'anrede', 'value' => $salutation )
			)
		);

		$result = $api->receiverAdd( v::vtglobal('pxtcCleverreachApiKey'), v::vtglobal('pxtcCleverreachListId'), $user );
		return $result->status == 'SUCCESS' ? $result->data->id : 'ERROR: Request via SoapClient failed';
	}

	require_once dirname( __FILE__ ) . '/cleverreach.class.php';
	$cleverreach = new Cleverreach( v::vtglobal('pxtcCleverreachId'), v::vtglobal('pxtcCleverreachApiKey'), v::vtglobal('pxtcSecureTransferMethod') );
	$data = array ( 'email' => $email, 'attributes' => compact( 'lastname', 'firstname', 'salutation' ) );
	return $cleverreach->addReceiver( v::vtglobal('pxtcCleverreachListId'), $data );
}


function myhtmlentities( $str )
{
	return htmlentities( $str, ENT_COMPAT | ENT_HTML401, 'UTF-8' );
}


function saveLightbox()
{
	if ( v::vtuser('id') >= 100 && v::vtuser('accountType') != 2 )
	{
		v::vtdo( array ( '-act' => 'update', '-obj' => v::vtuser('id'), 'Lightbox' => v::vtget('Lightbox') ) );
	}
	else
	{
		v::vtdo( array ( '-act' => 'setCookie', '-expires' => v::vtget('LightboxLifetime'), 'pxtc_lb1' => v::vtget('Lightbox') ) );
	}
}


function create_zip_file( $name, $basedir, $files )
{
	$files = array_map( 'escapeshellarg', array_map( 'trim', explode( '|', $files ) ) );
	$pxtc_dir = v::vtconfig('pathToVirthos');
	$cmd = v::vtconfig('pathToZipCommand') == '' ? '/usr/bin/zip' : v::vtconfig('pathToZipCommand');

	if ( ! ini_get('safe_mode') )
	{
		set_time_limit( 6*60*60 );
	}

	/*** Check/create temp directory ***/

	if ( v::vtconfig('pathToTemp') != '' && file_exists( v::vtconfig('pathToTemp') ) )
	{
		$tempdir = v::vtconfig('pathToTemp');
		$tempurl = v::vtconfig('baseURLTemp');
	}
	else
	{
		$tempdir = v::vtconfig('pathToData') . 'temp';

		if ( ! file_exists( $tempdir ) && ! @mkdir( $tempdir ) )
		{
			return -1; // could not create temp directory
		}

		chmod( $tempdir, 0777 );
		$tempurl = v::vtconfig('baseURLData') . 'temp/';
	}

	/*** Cleanup temp directory ***/

	$dh = opendir( $tempdir );

	while ( ( $file = readdir( $dh ) ) !== FALSE )
	{
		if ( $file != '.' && $file != '..' && is_dir( "$tempdir/$file" ) && filectime( "$tempdir/$file" ) < time() - 3*60*60 )
		{
			$dh2 = opendir( "$tempdir/$file" );

			while ( ( $file2 = readdir( $dh2 ) ) !== FALSE )
			{
				if ( $file2 != '.' && $file2 != '..' )
				{
					unlink( "$tempdir/$file/$file2" );
				}
			}

			closedir( $dh2 );
			rmdir( "$tempdir/$file" );
		}
	}

	/*** Create zip directory ***/

	for ( $i = 1; $i <= 20; $i++ )
	{
		$tempname = md5( microtime() . rand() );
		$zipdir = "$tempdir/$tempname";

		if ( mkdir( $zipdir, 0777 ) )
		{
			break;
			chmod( $zipdir, 0777 );
		}
	}

	if ( empty( $zipdir ) || ! file_exists( $zipdir ) )
	{
		return -2; // could not create temp subdirectory
	}

	/*** Create zip file ***/

	chdir( $basedir );
	$cmd .= " -0 $zipdir/$name " . implode( ' ', $files );
	exec( $cmd );

	if ( $handle = @opendir( $pxtc_dir . '/config/zip' ) )
	{
		$files = array ();

		while ( ( $filename = readdir( $handle ) ) !== FALSE )
		{
			if ( $filename[0] != '.' && is_file( $pxtc_dir . '/config/zip/' . $filename ) )
			{
				$files[] = escapeshellarg( $pxtc_dir . '/config/zip/' . $filename );
			}
		}

		if ( ! empty( $files ) )
		{
			$cmd .= " -0j $zipdir/$name " . implode( ' ', $files );
			exec( $cmd );
		}
	}

	return "$tempurl/$tempname/$name";
}


function deliver_zip_file( $name, $basedir, $files )
{
	if ( ! ini_get('safe_mode') )
	{
		set_time_limit( 12*60*60 );
	}

	require_once 'templates/pixtacy/lib/tbszip.php';
	$zip = new clsTbsZip();
	$zip->CreateNew();
	$files = json_decode( $files, TRUE );
	$pxtc_dir = v::vtconfig('pathToVirthos');

	foreach ( $files as $file )
	{
		if ( empty( $file ) )
		{
			continue;
		}

		if ( strpos( $file, '|' ) )
		{
			list ( $virtualPath, $file ) = explode( '|', $file );
			$zip->FileAdd( $virtualPath, $file, TBSZIP_FILE, FALSE );
		}
		else
		{
			$zip->FileAdd( $file, "{$basedir}/{$file}", TBSZIP_FILE, FALSE );
		}
	}

	if ( $handle = @opendir( "{$pxtc_dir}/config/zip" ) )
	{
		$files = array ();

		while ( ( $filename = readdir( $handle ) ) !== FALSE )
		{
			if ( $filename[0] != '.' && is_file( "{$pxtc_dir}/config/zip/{$filename}" ) )
			{
				$files[] = "{$pxtc_dir}/config/zip/{$filename}";
			}
		}

		if ( ! empty( $files ) )
		{
			foreach ( $files as $file )
			{
				$zip->FileAdd( basename( $file ), $file, TBSZIP_FILE, FALSE );
			}
		}
	}
	
	ignore_user_abort( TRUE );
	$countBuffers = 0;

	while ( ob_get_level() )
	{
		$countBuffers++;
		ob_end_clean();
	}

	$zip->Flush( TBSZIP_DOWNLOAD, $name, 'application/x-zip' );
	
	for ( $i = 0; $i <= $countBuffers; $i++ )
	{
		ob_start();
	}

	return ! connection_aborted();
}


function discountCart()
{
	/* berechnet alle Preise im Warenkorb neu unter Berücksichtigung der Produkt-Rabattstaffeln */

	$cart = json_decode( v::vtget('Cart'), TRUE );

	if ( empty( $cart ) )
	{
		v::vtset( array ( 'Cart' => '' ) );
		return;
	}

	$factor = v::vtuser('Preisfaktor') == '' ? 1 : (float) str_replace( ',', '.', v::vtuser('Preisfaktor') );
	$quantities = array ();
	$products = array ();

	foreach ( $cart as $item )
	{
		if ( isset( $quantities[ $item['product'] ] ) )
		{
			$quantities[ $item['product'] ] += $item['quantity'];
		}
		else
		{
			$quantities[ $item['product'] ] = $item['quantity'];
		}
	}

	foreach ( $quantities as $product_id => $quantity )
	{
		if ( v::vtuse( $product_id ) )
		{
			$price = (float) v::vtpage('Preis');

			if ( v::vtpage('Rabatt_anwenden') )
			{
				foreach ( json_decode( v::vtpage('Rabattstaffel'), TRUE ) as $level )
				{
					if ( $quantity >= $level['quantity'] )
					{
						$price = (float) $level['price'];
					}
				}
			}

			if ( v::vtpage('Preisanpassung') && $factor != 1 )
			{
				$price = $price * $factor;
			}

			$product['netprice'] = $price;
			$product['price'] = $price;
			$product['vat'] = 0;
			$product['vatrate'] = 0;

			if ( v::vtglobal('pxtcVatHandling') != '' )
			{
				$product['vatrate'] = v::vtpage('Steuersatz') == '2' ? v::vtglobal('pxtcVatRate2') : v::vtglobal('pxtcVatRate');

				if ( v::vtglobal('pxtcVatHandling') == 'excluded' )
				{
					$product['netprice'] = $price;
					$product['vat'] = round( $price * $product['vatrate']/100, 2 );
					$product['price'] = $product['netprice'] + $product['vat'];
				}
				else
				{
					$product['price'] = $price;
					$product['netprice'] = round( $price / (1 + $product['vatrate']/100 ), 2 );
					$product['vat'] = $product['price'] - $product['netprice'];
				}
			}

			$products[ $product_id ] = $product;
		}

		v::vtenduse();
	}

	foreach ( $cart as $item )
	{
		if ( isset( $products[ $item['product'] ] ) )
		{
			$product = $products[ $item['product'] ];
			$item['price'] = $product['price'];
			$item['netprice'] = $product['netprice'];
			$item['vat'] = $product['vat'];
			$item['vatrate'] = $product['vatrate'];
			$item['total'] = $product['price'] * $item['quantity'];
			$item['nettotal'] = $product['netprice'] * $item['quantity'];
		}

		$newcart[] = $item;
	}

	v::vtset( array ( 'Cart' => json_encode( $newcart ) ) );
	return $newcart;
}


function get_cart_price( $product_id )
{
	static $quantities;

	if ( ! v::vtuse( $product_id ) || v::vtpage('vtPageType') != 'product' )
	{
		return FALSE;
	}

	if ( ! isset( $quantities ) )
	{
		$cart = json_decode( v::vtget('Cart'), TRUE );
		$quantities = array ();

		foreach ( $cart as $item )
		{
			$quantities[ $item['product'] ] = isset( $quantities[ $item['product'] ] ) ? $quantities[ $item['product'] ] + $item['quantity'] : $item['quantity'];
		}
	}

	$quantity = isset( $quantities[ $product_id ] ) ? $quantities[ $product_id ] : 1;
	$price = (float) v::vtpage('Preis');
	$factor = v::vtuser('Preisfaktor') == '' ? 1 : (float) str_replace( ',', '.', v::vtuser('Preisfaktor') );

	if ( v::vtpage('Rabatt_anwenden') )
	{
		foreach ( json_decode( v::vtpage('Rabattstaffel'), TRUE ) as $level )
		{
			if ( $quantity >= $level['quantity'] )
			{
				$price = (float) $level['price'];
			}
		}
	}

	if ( v::vtpage('Preisanpassung') && $factor != 1 )
	{
		$price = $price * $factor;
	}

	$product['netprice'] = $price;
	$product['price'] = $price;

	if ( v::vtglobal('pxtcVatHandling') != '' )
	{
		$vatrate = v::vtpage('Steuersatz') == '2' ? v::vtglobal('pxtcVatRate2') : v::vtglobal('pxtcVatRate');

		if ( v::vtglobal('pxtcVatHandling') == 'excluded' )
		{
			$product['netprice'] = $price;
			$product['price'] = $product['netprice'] + $product['vat'];
		}
		else
		{
			$product['price'] = $price;
			$product['netprice'] = round( $price / (1 + $vatrate/100 ), 2 );
		}
	}

	v::vtenduse();
	return v::vtglobal('pxtcVatHandling') == 'excluded' ? $product['netprice'] : $product['price'];
}


function get_cart_quantity( $image_id, $product_id )
{
	static $quantities;

	if ( ! isset( $quantities ) )
	{
		$cart = json_decode( v::vtget('Cart'), TRUE );
		$quantities = array ();

		foreach ( $cart as $item )
		{
			$quantities[ $item['image'] ][ $item['product'] ] = $item['quantity'];
		}
	}

	if ( isset( $quantities[ $image_id ][ $product_id ] ) )
	{
		return $quantities[ $image_id ][ $product_id ];
	}

	return 0;
}


function xml_character_encode( $string )
{
	$trans = get_html_translation_table( HTML_ENTITIES, ENT_QUOTES );

	foreach ( $trans as $key => $value )
	{
		$trans[ $key ] = '&#' . ord( $key ) . ';';
	}

	return strtr( $string, $trans );
}


function create_sepa_mandate_ref()
{
	list ( $micro, $time ) = explode( ' ', microtime() );
	$code = base_convert( $time . substr( $micro, 2 ), 10, 34 );
	return v::vtglobal('pxtcSepaRefPrefix') . strtoupper( strtr( $code, 'o0', 'yz' ) );
}


function rmdir_recursive( $dir )
{
	$objects = scandir( $dir );

	foreach ( $objects as $object )
	{
		if ( $object == '.' || $object == '..' )
		{
			continue;
		}

		$function = is_dir( $dir.'/'.$object ) ? 'rmdir_recursive' : 'unlink';
		$function( $dir.'/'.$object );
	}

	rmdir( $dir );
}


function update_lores_image( $image_path, $force = FALSE )
{
	require_once 'templates/pixtacy/lib/image_processor.class.php';
	$hires_file = v::vtglobal('pxtcPathHires') . $image_path;

	if ( ! file_exists( $hires_file ) )
	{
		$hires_file = substr( $hires_file, 0, -3 ) . 'JPG';

		if ( ! file_exists( $hires_file ) )
		{
			return -1; // High-res file not found
		}
	}

	$path = v::vtglobal('pxtcPathLores') . dirname( $image_path );

	if ( file_exists( $path ) )
	{
		if ( ! is_dir( $path ) )
		{
			return -10; // Cannot create Low-res directory
		}
	}
	else
	{
		if ( ! mkdir( $path, 0777, TRUE ) )
		{
			return -11; // Creation of Low-res directory failed
		}
		
		chmod( $path, 0777 );
	}

	if ( ! is_writable( $path ) )
	{
		v::vtdo( array ( '-act' => 'ftp', '-cmd' => 'chmod', '-remote' => convertToFtpPath( $path ), '-mode' => '0777' ) );

		if ( v::vtresult() != 1 )
		{
			return -9; // Low-res directory not writable
		}
	}

	$path = v::vtglobal('pxtcPathLores') . $image_path;

	if ( file_exists( $path ) && ! is_writable( $path ) )
	{
		v::vtdo( array ( '-act' => 'ftp', '-cmd' => 'delete', '-remote' => convertToFtpPath( $path ) ) );

		if ( v::vtresult() != 1 )
		{
			return -8; // Low-res file exists and cannot be overwritten
		}
	}

	if ( file_exists( $path ) && filemtime( $image_path ) >= filemtime( $path ) && ! $force )
	{
		return 0; // Low-res file is newer than High-res file
	}

	$imager = new Image_Processor( array ( 'watermarkMargin' => max( 2, v::vtglobal('pxtcImagingLoresWatermarkMargin') ) ) );

	if ( ! empty( v::vtglobal('pxtcXmpSupport') ) )
	{
		require_once 'templates/pixtacy/lib/image/ImageInterface.php';
		require_once 'templates/pixtacy/lib/image/Image.php';
		require_once 'templates/pixtacy/lib/image/Metadata/Xmp.php';
		require_once 'templates/pixtacy/lib/image/Format/JPEG.php';
		require_once 'templates/pixtacy/lib/image/Format/JPEG/Segment.php';
		$imager->xmpSupport = TRUE;
	}

	if ( ! $imager->load( $hires_file, TRUE ) )
	{
		return -2; // Not enough memory
	}

	$size = v::vtglobal('pxtcImagingLoresSize');
	$sizeMin = v::vtglobal('pxtcImagingLoresSizeMin');
	$imager->resize( $size ? $size : 800, $sizeMin ? $sizeMin : 0 );

	$size = v::vtglobal('pxtcImagingLoresWatermarkSize') ? v::vtglobal('pxtcImagingLoresWatermarkSize') : 0.7;
	$color = v::vtglobal('pxtcImagingLoresWatermarkColor') ? v::vtglobal('pxtcImagingLoresWatermarkColor') : 'ffffff';

	if ( v::vtglobal('pxtcImagingLoresWatermark') == 'image' )
	{
		$imager->add_watermark_image( v::vtglobal('pxtcImagingLoresWatermarkUrl'), v::vtglobal('pxtcImagingLoresWatermarkPos'), v::vtglobal('pxtcImagingLoresWatermarkOpacity'), $size );
	}
	elseif ( v::vtglobal('pxtcImagingLoresWatermark') == 'text' && v::vtglobal('pxtcImagingLoresWatermarkText') != '' )
	{
		$imager->add_watermark_text( v::vtglobal('pxtcImagingLoresWatermarkText'), v::vtglobal('pxtcImagingLoresWatermarkPos'), v::vtglobal('pxtcImagingLoresWatermarkOpacity'), $size, $color, NULL, NULL, v::vtglobal('pxtcImagingLoresWatermarkShadow') );
	}

	$quality = v::vtglobal('pxtcImagingLoresQuality');

	if ( ! $imager->write( $path, $quality ? $quality : 80 ) )
	{
		return -12;
	}

	return 1;
}


function getClassKeywords( $keywordList )
{
	$result = preg_match_all( '/(\A|,\s*)\.\@([^,]+)/', $keywordList, $matches );
	return $result ? implode( ' ', $matches[2] ) : '';
}


function initializeFlags()
{
	$flags = isset( $_COOKIE['pxtc_flags'] ) ? json_decode( $_COOKIE['pxtc_flags'], TRUE ) : array();

	if ( v::vtuser('id') >= 100 && v::vtuser('accountType') != 2 )
	{
		if ( empty( $flags ) )
		{
			$flags = json_decode( v::vtuser('Flags'), TRUE );
		}
		elseif ( v::vtuser('Flags') != '' )
		{
			$cookieFlags = $flags;
			$flags = json_decode( v::vtuser('Flags'), TRUE );

			foreach ( $cookieFlags as $imageId => $imageFlags )
			{
				foreach ( str_split( $imageFlags ) as $flag )
				{
					if ( empty( $flags[ $imageId ] ) )
					{
						$flags[ $imageId ] = $flag;
					}
					elseif ( strpos( $flags[ $imageId ], $flag ) === FALSE )
					{
						$flags[ $imageId ] .= $flag;
					}
				}
			}

			v::vtdo( 'update', array( '-obj' => v::vtuser('id'), 'Flags' => json_encode( $flags ) ) );
			v::vtdo( 'setCookie', array( '-expires' => -1, 'pxtc_flags' => '' ) );
		}
	}

	v::vtset( array( 'Flags' => json_encode( $flags ) ) );
}


function getFlagsForPage( $pageId, $asClassNameList = FALSE )
{
	static $flags;

	if ( ! isset( $flags ) )
	{
		$flags =  v::vtget('Flags') == '' ? array() : json_decode( v::vtget('Flags'), TRUE );
	}

	if ( empty( $flags[ $pageId ] ) )
	{
		return $asClassNameList ? '' : array();
	}

	$pageFlags = str_split( $flags[ $pageId ] );

	if ( $asClassNameList )
	{
		return 'pxtc-flag-' . implode( ' pxtc-flag-', $pageFlags );
	}
	else
	{
		return $pageFlags;
	}
}


function readFolderInfo( $folder )
{
	$file = rtrim( $folder, '/' ) . '/_pxtc.ini';
	$info = file_exists( $file ) ? parse_ini_file( $file ) : FALSE;
	return $info ? json_encode( $info ) : '{}';
} 


function generate_image_file( $filePath, $imagingRule, $pageId = NULL )
{
	static $imager, $rules;

	if ( ! file_exists( $filePath ) )
	{
		return -1; // File not found
	}

	if ( v::vtglobal('pxtcImagingRules') == '' )
	{
		return -2; // No imaging rules defined
	}

	if ( ! isset( $rules ) )
	{
		$rules = json_decode( v::vtglobal('pxtcImagingRules,raw'), TRUE );
	}

	foreach ( $rules as $item )
	{
		if ( $item['name'] == $imagingRule )
		{
			$rule = $item;
			break;
		}
	}

	if ( empty( $rule ) )
	{
		return -3; // Imaging rule not found
	}
	
	if ( ! isset( $imager ) )
	{
		require_once 'templates/pixtacy/lib/image_processor.class.php';
		$imager = new Image_Processor();
	}

	if ( ! $imager->load( $filePath, TRUE ) )
	{
		return -4; // Not enough memory
	}

	if ( $rule['size'] > 0 )
	{
		$imager->resize( $rule['size'], $rule['minimum'] );

		if ( $pageId && v::vtuse( $pageId ) )
		{
			$imager->set_metadata( array (
				 '2#015' => v::vtpage('Category'),
				 '2#020' => v::vtpage('SupCategory'),
				 '2#025' => prepare_keywords_for_export( v::vtpage('Keywords') ),
				 '2#050' => v::vtpage('ReferenceNumber'),
				 '2#055' => v::vtpage('CreationDate'),
				 '2#060' => v::vtpage('CreationTime'),
				 '2#080' => v::vtpage('Photographer'),
				 '2#080' => v::vtpage('ByLine'),
				 '2#085' => v::vtpage('ByLineTitle'),
				 '2#090' => v::vtpage('City'),
				 '2#095' => v::vtpage('State'),
				 '2#100' => v::vtpage('CountryCode'),
				 '2#101' => v::vtpage('Country'),
				 '2#110' => v::vtpage('Credit'),
				 '2#105' => v::vtpage('Name'),
				 '2#116' => v::vtpage('Copyright'),
				 '2#120' => strip_tags( v::vtpage('Remarks') ),
				 '2#040' => v::vtpage('Instructions'),
				 '2#122' => v::vtpage('Writer')
			), TRUE );
			v::vtenduse();
		}
	}

	if ( isset( $rule['watermark'] ) && $rule['watermark'] == 'text' )
	{
		$imager->add_watermark_text( $rule['wmtext'], $rule['wmpos'], $rule['wmopacity'], $rule['wmsize'], $rule['wmcolor'], NULL, NULL, $rule['wmshadow'] );
	}
	elseif ( isset( $rule['watermark'] ) && $rule['watermark'] == 'image' )
	{
		$imager->add_watermark_image( $rule['wmurl'], $rule['wmpos'], $rule['wmopacity'], $rule['wmsize'] );
	}

	$tempfile = v::vtconfig('pathToCache') . 'images/' . sha1( $filePath . '-' . $rule['name'] ) . '.jpg' ;
	
	if ( ! $imager->write( $tempfile, $rule['quality'] ) )
	{
		return -5; // Could not write temporary file
	}
	
	return $tempfile;
}


function prepare_keywords_for_export( $keywords )
{
	$keywords = array_map( 'trim', explode( ',', $keywords ) );
	return array_filter( $keywords, function( $var ){ return $var != '' && $var[0] != '.'; } );
}


function hires_file_exists( $imagePath, $code )
{
	static $listings;
	$folder = v::vtglobal('pxtcPathHires') . dirname( $imagePath )
		. '/' . ( $code == '*' ? '' : $code );
	if ( ! is_dir( $folder ) ) {
		return FALSE;
	}
	if ( ! isset( $listings[ $folder ] ) ) {
		foreach ( scandir( $folder ) as $item ) {
			if ( $item[0] != '.' ) {
				$listing[] = pathinfo( $item, PATHINFO_FILENAME );
			}
		}
		$listings[ $folder ] = array_unique( $listing );
	}
	return in_array( pathinfo( $imagePath, PATHINFO_FILENAME ), $listings[ $folder ] );
}


function image_is_buyable( $imagePath, $imageAssortment, $imageProducts, $returnProductList = FALSE )
{
	static $allImagesBuyable = FALSE;
	static $noImageBuyable = FALSE;
	static $globalProducts; // Gesamtliste der theoretisch kaufbaren Produkte
	static $assortmentCodes; // Gesamtliste der Sortimente ( Seiten-ID = Schlüssel, Kennungen = Wert )
	static $productAssortments; // Liste der Sortimente, denen ein Produkt (ID = Schlüssel) zugeordnet ist

	if ( $allImagesBuyable && ! $returnProductList ) {
		return TRUE;
	}

	if ( $noImageBuyable ) {
		return FALSE;
	}

	if ( is_null( $productAssortments ) ) {
		$sql = "SELECT c.`id`, a.`liste` 
			FROM `%data_assortment` a LEFT JOIN `%content` c ON c.id = a.vtid 
			WHERE c.containerID = " . (int) v::vtget('ID_Sortimente');
		$result = v::query( $sql );

		while ( $assortment = $result->fetch_assoc() ) {
			$elements = json_decode( $assortment['liste'], TRUE );
			foreach ( $elements as $element ) {
				if ( ! empty( $element['product_id'] ) ) {
					$productAssortments[ $element['product_id'] ][] = $assortment['id'];
				}
			}
		}
	}

	if ( is_null( $globalProducts ) ) {
		$paymentMethods = v::vtget('Bezahlmethoden');
		$deliveryMethods = v::vtget('DeliveryMethods');
		$deliveryPayment = v::vtget('DeliveryPayment');
		$groups = explode( ',', v::vtuser('groups') );
		$sql = "SELECT c.id, p.`benutzer`, p.`kennung`, p.`preis` + 0 as 'preis', p.`zuordnung`, p.`typ` 
			FROM `%data_product` p LEFT JOIN `%content` c ON c.id = p.vtid 
			WHERE c.containerID = " . (int) v::vtget('ID_Produkte');
		$result = v::query( $sql );

		while ( $product = $result->fetch_assoc() ) {
			if ( ( empty( $paymentMethods ) && $product['preis'] && empty( $deliveryPayment ) )
				|| ( empty( $deliveryMethods ) && strtolower( $product['typ'] ) == 'print' )
				|| ( ! empty( $product['benutzer'] ) && ! in_array( $product['benutzer'], $groups ) 
				&& ! ( $product['benutzer'] == 'guest' && v::vtuser('name') == '' )  
				&& ! ( $product['benutzer'] == 'Alle' && v::vtuser('name') != '' ) ) 
			) {
				continue;
			}
			$prodId = (int) $product['id'];
			$globalProducts[ $prodId ] = $product;
			if ( $product['zuordnung'] ) {
				// es ist ein automatisch zugeordnetes Produkt
				if ( empty( $product['kennung'] ) ) {
					// es ist nicht von High-res-Datei abhängig
					$allImagesBuyable = TRUE;
					if ( ! $returnProductList ){
						return TRUE;
					}
				}
				// von High-res-Datei abhängig - Kennung in automatisches Sortiment aufnehmen
				if ( empty( $assortmentCodes[0] ) || ! in_array( $product['kennung'], $assortmentCodes[0] ) ){
					$assortmentCodes[0][] = $product['kennung'];
				}
			}
			elseif ( ! isset( $productAssortments[ $prodId ] ) ) { 
				// Produkt ist keinem Sortiment zugeordnet
				continue;
			}
			elseif ( empty( $product['kennung'] ) ) { 
				// Produkt ist nicht von High-res-Datei abhängig
				// setze alle Sortimente, denen das Produkt zugeordnet ist, auf "generell kaufbar"
				foreach ( $productAssortments[ $prodId ] as $assortmentId ) {
					$assortmentCodes[ $assortmentId ] = '~';
				}
			}
			else {
				// Produkt ist von High-res-Datei abhängig
				// nimm Kennung in Kennungsliste der Sortimente auf, denen das Produkt zugeordnet ist
				foreach ( $productAssortments[ $prodId ] as $assortmentId ) {
					if ( empty( $assortmentCodes[ $assortmentId ] ) || ( $assortmentCodes[ $assortmentId ] != '~' 
						&& ! in_array( $product['kennung'], $assortmentCodes[ $assortmentId ] ) ) 
					) {
						$assortmentCodes[ $assortmentId ][] = $product['kennung'];
					}
				}
			}
		}
		if ( empty( $globalProducts ) ) {
			$noImageBuyable = TRUE;
			return FALSE;
		}
	}

	// Prüfe Bild individuell

	if ( $imageAssortment == -1 || ! isset( $assortmentCodes[ $imageAssortment ] ) ) {
		// kein Sortiment zugeordnet
		// prüfe automat. zugeordnete Produkte
		if ( ! empty( $assortmentCodes[0] ) ){
			foreach ( $assortmentCodes[0] as $code ){
				if ( hires_file_exists( $imagePath, $code ) ){
					if ( ! $returnProductList ){
						return TRUE;
					}
				}
			}
		}
		// prüfe individuell zugeordnete Produkte
		if ( empty( $imageProducts ) && ! $returnProductList ) {
			return FALSE;
		}
		foreach ( explode( "\n", $imageProducts ) as $product ) {
			if ( isset( $globalProducts[ $product ] ) 
				&& ( $globalProducts[ $product ]['kennung'] == '' || hires_file_exists( $imagePath, $globalProducts[ $product ]['kennung'] ) )
			) {
				if ( ! $returnProductList ){
					return TRUE;
				}
			}
		}
		return FALSE;
	}

	if ( $assortmentCodes[ $imageAssortment ] == '~' ) {
		// zugeordnetes Sortiment ist generell kaufbar
		if ( ! $returnProductList ){
			return TRUE;
		}
	}
	
	// zugeordnetes Sortiment ist nicht generell kaufbar - prüfe High-res-Dateien
	foreach ( $assortmentCodes[ $imageAssortment ] as $code ) {
		if ( hires_file_exists( $imagePath, $code ) ) {
			if ( ! $returnProductList ){
				return TRUE;
			}
		}
	}

	return $returnProductList ? $imageProducts : FALSE;
}

?>
