/*! lg-cart - v0.9 - 2023-01-14
* Copyright (c) 2023 Martin Wandelt */

(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.lgCart = factory());
}( this, ( function () { 'use strict';

	var __assign = function() {
		__assign = Object.assign || function __assign(t) {
			for (var s, i = 1, n = arguments.length; i < n; i++) {
				s = arguments[i];
				for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
			}
			return t;
		};
		return __assign.apply(this, arguments);
	};

	var cartSettings = {
		cart: true,
	};

	/**
	* List of lightGallery events
	* All events should be documented here
	* Below interfaces are used to build the website documentations
	* */
	var lGEvents = {
		afterAppendSlide: 'lgAfterAppendSlide',
		init: 'lgInit',
		hasVideo: 'lgHasVideo',
		containerResize: 'lgContainerResize',
		updateSlides: 'lgUpdateSlides',
		afterAppendSubHtml: 'lgAfterAppendSubHtml',
		beforeOpen: 'lgBeforeOpen',
		afterOpen: 'lgAfterOpen',
		slideItemLoad: 'lgSlideItemLoad',
		beforeSlide: 'lgBeforeSlide',
		afterSlide: 'lgAfterSlide',
		posterClick: 'lgPosterClick',
		dragStart: 'lgDragStart',
		dragMove: 'lgDragMove',
		dragEnd: 'lgDragEnd',
		beforeNextSlide: 'lgBeforeNextSlide',
		beforePrevSlide: 'lgBeforePrevSlide',
		beforeClose: 'lgBeforeClose',
		afterClose: 'lgAfterClose',
		rotateLeft: 'lgRotateLeft',
		rotateRight: 'lgRotateRight',
		flipHorizontal: 'lgFlipHorizontal',
		flipVertical: 'lgFlipVertical',
		autoplay: 'lgAutoplay',
		autoplayStart: 'lgAutoplayStart',
		autoplayStop: 'lgAutoplayStop',
	};

	var Cart = ( function( element ){

		function Cart( instance, $LG ) {
			// get lightGallery core plugin instance
			this.core = instance;
			this.$LG = $LG;
			this.settings = __assign(__assign({}, cartSettings), this.core.settings);
			return this;
		}

		Cart.prototype.init = function() {

			if ( ! this.settings.cart ) {
				return;
			}

			var sheet = document.createElement('style')
			sheet.innerHTML = 
				'.lg-cart-box { padding:20px; z-index:9999; position:absolute; cursor:pointer; } ' +
				'.lg-cart-box i { color:white; opacity:0.9; text-shadow: 1px 1px 2px black, 0 0 1em grey, 0 0 0.2em grey; font-size: 1.75rem } ' +
				'.in-cart.lg-cart-box i { color:orange; } ' +
				'.lg-cart-box:hover i { opacity:1; text-shadow: 1px 1px 2px black, 0 0 1em black, 0 0 0.2em black; } ' +
				'.lg-cart-dialog { position:fixed; top:0; right:0; width:300px; height:100%; background:white; z-index:99999; }';
			document.head.appendChild( sheet );

			var _this = this;
			var html = '<div class="lg-cart-box position-absolute p-4"><i class="fas fa-shopping-cart"></i></div>';
			this.core.outer.find('.lg-content').append( html );
			this.cartbox = this.core.outer.find('.lg-cart-box');
			window.setTimeout( function(){ _this.adjustBoxPosition(); }, 1000 );

			$(window).on( 'resize', function(){
				_this.adjustBoxPosition();
			});

			this.core.LGel.on( lGEvents.afterSlide, function( event, prevIndex, index ){
				_this.core.currentIndex = index;
				_this.updateBadge();
			});

			this.core.LGel.on( lGEvents.beforeSlide, function( event, prevIndex, index ){
				_this.cartbox.css( 'display', 'none' );
				_this.cartbox.removeClass('in-cart');
			});

			this.core.LGel.on( lGEvents.slideItemLoad, function(){
				_this.updateBadge();
			});

			this.core.LGel.on( lGEvents.posterClick, function(){
				_this.updateBadge();
			});

			this.core.outer.find('.lg-cart-box').first().on( 'click', function(){
				var elm = $( '#' + _this.getAttribute('elementId') );
				var url = elm.data('productlist');
				if ( url ){
					$('#add-to-cart').load( url, function(){
						if ( $(this).find('form[name=anmeldung]').length ){
							window.location.reload();
							return;
						}
						$('#add-to-cart').modal('show');
					});
				}
			});
		};

		Cart.prototype.updateBadge = function(){
			var elm = $( '#' + this.getAttribute('elementId') );
			if ( elm.hasClass('in-cart') ){
				this.cartbox.addClass('in-cart');
			}
			this.adjustBoxPosition();
			this.cartbox.css( 'display', 'block' );
		}

		Cart.prototype.adjustBoxPosition = function(){
			var outerPos = this.core.outer.offset();
			var pos = this.core.outer.find('.lg-current .lg-image').offset();
			var toolbarHeight = this.core.$toolbar.firstElement.offsetHeight;
			if ( pos ){
				this.cartbox.css( 'top', '' + ( pos.top - outerPos.top - toolbarHeight ) + 'px' );
				this.cartbox.css( 'right', '' + pos.left + 'px' );
			}
		}

		Cart.prototype.getAttribute = function( prop ){
			var index = this.core.index;
			return this.core.galleryItems[ index ][ prop ];
			//return this.core.$items.eq( index ).data( prop );
		}

		Cart.prototype.buy = function(){
			var url = this.getAttribute('productList');
			if ( url ) {
				$('.modal-body').load( url, function(){
					$('#add-to-cart').modal('show');
				});
			}
		};
		
		Cart.prototype.showDialog = function(){
			this.core.$outer.css( 'width', '' + ( this.core.$outer.width() - 300 ) + 'px' )
			.find('.lg-sub-html').css( 'right', '300px' );
			var _this2 = this;
			var html = '<div class="lg-cart-dialog"><button>close</button></div>';
			$('body').first().append( html );
			$('.lg-cart-dialog').height( this.core.$outer.height() );
			$('.lg-cart-dialog button').on( 'click', function(){
				_this2.hideDialog();
			});
		}

		Cart.prototype.hideDialog = function(){
			this.core.$outer.css( 'width', '100%' )
			.find('.lg-sub-html').css( 'right', '0' );
			$('.lg-cart-dialog').remove();
		}

		Cart.prototype.destroy = function() {
			this.hideDialog();
			$('#add-to-cart').modal('hide');
		};

		return Cart;
	}());

	return Cart;
})));

